/*
==============================================================
Ubershader usable parameters:
-----------------------------
Guide:
   +	: optional parameter (the previous one should be added first)
   ++ 	: mandatory parameter (one of them must be added if the previous one is added first)
   -or-	: exclusive parameter (if the element it's added noone of the similar/neighbour params can be added)
-----------------------------
USE_FOG -> the shader use fog
	+ USE_VFOG -> the shader will use vertical fog instad of the classical fog
	
USE_EMISSIVENESS -> the G channel of the MK texture is the mask for the emissiven (self illuminated) parts of the object

USE_DIFFUSELIGHT -> adds diffuse-perPixelLighting (no specular, only diffuse -> NdotL)

USE_LINEARSPACE_DF -> corrects the NdotL calculation to linear-space (pow 0.45)

USE_SPECULARLIGHT -> adds specular light to the object
	++ USE_SPECULARLIGHT0 -> specPerLight (specular computed per light position + specularGlossiness)
	++ USE_SPECULARLIGHT1 -> specPalettePerLight (specular computed per light position + specularPalette [Fx_SPEC_xxxxx.tga])
	++ USE_SPECULARLIGHT2 -> specPerView (specular computed per camera + specularGlossiness)
	++ USE_SPECULARLIGHT3 -> specPalettePerView (specular computed per camera + specularPalette [Fx_SPEC_xxxxx.tga])
	
USE_LINEARSPACE_SP -> corrects the specular calculation to linear-space (pow 0.45)
	
USE_SPECULARMASK -> the R channel of the MK texture is the mask for the specular (specular level) of the objects. If this is not active, the specular mask is computed per luminance intensity of the diffuse texture.

USE_NDIS -> adds the NDIS texture as specular (thid means: additive over the diffuse). If there is specualr too, both are added.
 -or-
USE_NDISREFLECTION -> adds the NDIS texture as reflection (thid means: mix over the diffuse).
	+ USE_NDISREFRACTION -> compute the fake reflection as refraction for any of the two previous NDIS.
	
USE_CH3DECALS -> multiply a decalMap (mapped using the 3dsmax UVW-channel#3 / shader uv2) over the diffuse texture.
-or-
USE_CH3DECALSMIX -> mix by decal-alpha a decalMap (mapped using the 3dsmax UVW-channel#3 / shader uv2) over the diffuse texture.
-or-
USE_CH3ALPHA -> use the R channel of a texture called "alphaMap" (mapped with the uvw ch3) as alpha

USE_VCOLOR_MIX -> mix a second set of textures (DF2 + NM2 + MK2)
	++ USE_VCOLOR_MIX0	-> mix the second set of textures with the VC.r
	-or-
	++ USE_VCOLOR_MIX1 -> mix the second set of textures with a bitmap mask: (dirtMixMap)
-or-
USE_VCOLOR_MASKB_TINT -> multiply the vertex color over the diffuse modulated with the B channel of the MK texture
-or-
USE_VCOLOR_ALPHA -> use the R channel of then VC as alpha channel.
-or-
USE_VCOLOR_LM -> use the vertex color as LM

USE_SNOWCOVERED -> mix a snowMap using a mask texture (edgeMap). both textures use a custom uv map coordinates

USE_ANIMATEDOVERLAY -> add an animated overlay over the material
-or-
USE_ANIMATEDOVERLAY_CAUSTIC -> add an animated overlay distorted with a normalmap over the material

USE_VERTEXLIGHT -> use vertex lighting calculated with a maximum of 5 simultaneous lights. The ligh is applied with additive 

USE_SHADOWMAP ++ USE_SHADOWS -> use shadowmapping.

==============================================================
Define parameters (example):
----------------------------
#define USE_FOG	
#define USE_VFOG					//<-- optional for [USE_FOG]
#define USE_EMISSIVENESS
	//#define USE_DIFFUSELIGHT
#define USE_SPECULARLIGHT
#define USE_SPECULARLIGHT0			//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT1	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT2	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT3	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_LINEARSPACE_DF
	//#define USE_LINEARSPACE_SP
#define USE_SPECULARMASK
	//#define USE_NDIS				//<-- exclusive between [USE_NDIS] & [USE_NDISREFLECTION]
#define USE_NDISREFLECTION			//<-- exclusive between [USE_NDIS] & [USE_NDISREFLECTION]
	//#define USE_NDISREFRACTION 	//<-- optional for [USE_NDIS] or [USE_NDISREFLECTION]
	//#define USE_CH3DECALS			//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_CH3DECALSMIX		//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_CH3ALPHA			//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_VCOLOR_MIX		//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_MIX0		//<-- mandatory for [USE_VCOLOR_MIX]
	//#define USE_VCOLOR_MIX1		//<-- mandatory for [USE_VCOLOR_MIX]
	//#define USE_VCOLOR_MASKB_TINT	//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_ALPHA		//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_LM			//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_SNOWCOVERED
#define USE_ANIMATEDOVERLAY
	//#define USE_ANIMATEDOVERLAY_CAUSTIC //<-- optional for [USE_ANIMATEDOVERLAY]
#define USE_VERTEXLIGHT
#define USE_SHADOWMAP
#define USE_SHADOWS					//<-- mandatory for [USE_SHADOWMAP]
==============================================================
*/

uniform sampler2D diffuseMap;

uniform lowp float overbright;

varying highp   vec2  uv0;	

#if defined(USE_ANIMATEDOVERLAY_CAUSTIC)
	uniform sampler2D causticNormalMap;
	uniform lowp float distorsionAmount;
	uniform highp vec2 overlayUVs;
#endif

#if defined(USE_ANIMATEDOVERLAY) || defined(USE_ANIMATEDOVERLAY_CAUSTIC)
	uniform sampler2D overlayMap;
	varying highp	vec2 uvOverlay;
#endif

#if !defined (USE_VCOLOR_LM)
	uniform sampler2D lightMap;
	uniform lowp float brightBurn; //<-- can be eliminated for optimization purpose
	varying highp   vec2  uv1;
#endif

#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS) || defined(USE_NDISREFLECTION)
	uniform sampler2D normalMap;
#endif

#if defined(USE_DIFFUSELIGHT)
	uniform lowp float DFLIntensity;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined(USE_VCOLOR_MIX) || defined (USE_VCOLOR_ALPHA) || defined (USE_VCOLOR_LM)
	varying lowp 	vec4  	VC;
#endif

#if defined(USE_CH3DECALS)
	uniform sampler2D decalMap;
#endif

#if defined(USE_VCOLOR_MIX) || defined(USE_CH3ALPHA) || defined(USE_CH3DECALS)
	varying highp   vec2  uv2;
#endif

#if defined(USE_CH3ALPHA)
	uniform sampler2D alphaMap;
#endif

#if defined(USE_VCOLOR_MIX)
	uniform sampler2D diffuseMap2;
	#if defined(USE_VCOLOR_MIX1)
		uniform sampler2D dirtMixMap;
		uniform lowp float dirtMixIntensity;
		uniform lowp float dirtMixCuttOff;
		varying highp   vec2  uvDirt;
	#endif
#endif

#if defined(USE_SNOWCOVERED)
	uniform sampler2D snowMap;
	uniform sampler2D edgeMap;
	uniform	mediump float snowThreshold;
	uniform	mediump float snowIntensity;
	varying highp   vec4  uvSnow;
	varying lowp   	float  snowMask;
#endif

#if defined(USE_VCOLOR_MIX)
	#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS) || defined(USE_NDISREFLECTION)
		uniform sampler2D maskMap2;	
	#endif
	#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS) || defined(USE_DIFFUSELIGHT) || defined(USE_NDISREFLECTION)
		uniform sampler2D normalMap2;
	#endif
#endif

#if defined(USE_FOG)
	uniform lowp 	vec4 		inFogColor;
	varying lowp 	float 		fogRatio;
#endif

#if defined(USE_EMISSIVENESS) || defined(USE_SPECULARMASK) || defined(USE_VCOLOR_MIX) || defined(USE_VCOLOR_MASKB_TINT)
	uniform sampler2D maskMap;
#endif

#if defined(USE_DIFFUSELIGHT)
	varying lowp vec3  LightDirection;
#endif

#if defined(USE_SPECULARLIGHT)
	uniform mediump float specularIntensity;
	uniform mediump vec3 inPLightDF[8];
	varying lowp vec3  ViewDirection;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT0) || defined(USE_SPECULARLIGHT2)
	uniform mediump float specularGlossy;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT0) || defined(USE_SPECULARLIGHT1)
	varying lowp vec3  H;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT1) || defined(USE_SPECULARLIGHT3)
	uniform sampler2D specularPalette;
#endif				

#if defined(USE_VERTEXLIGHT)
	varying lowp vec4  VXL;
#endif

#if defined(USE_NDIS) || defined(USE_NDISREFLECTION)
	uniform 		sampler2D 	reflectionMap;
	uniform lowp 	float 		reflectionAmount;
	varying highp 	vec2  		VSN;
#endif

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	uniform lowp sampler2D shadowMap;
	uniform lowp float shadowMapOpacity;
	varying highp vec2 lightProjectedUV;
	//varying highp float lightProjectedZ;	
#endif

#if defined(USE_COLORCORRECTION)	
	uniform lowp mat4 g_ColorCorrectionMatrix;
#endif

void main()
{					
	lowp vec4 DF 	= jetAlphaTexture2D(diffuseMap 	, uv0);
	
	#if !defined (USE_VCOLOR_LM)
		lowp vec4 LM 	= texture2D(lightMap 	, uv1);
	#endif	
	
	#if defined(USE_CH3DECALS) || defined(USE_CH3DECALSMIX)
		lowp vec4 DECAL 	= jetAlphaTexture2D(decalMap 	, uv2);
	#endif	
	
	#if defined(USE_CH3ALPHA)
		lowp float ALPHA 	= texture2D(alphaMap 	, uv2).r;
	#endif
	
	#if defined(USE_VCOLOR_MIX) && defined(USE_VCOLOR_MIX0)
		lowp float mixVal = VC.r;
	#endif
	
	#if defined(USE_VCOLOR_MIX) && defined(USE_VCOLOR_MIX1)
		lowp vec4  DM = texture2D(dirtMixMap, uvDirt);
		lowp float mixVal = min(pow((VC.r * DM.r * dirtMixIntensity), dirtMixCuttOff),1.0);
	#endif
		
	#if defined(USE_VCOLOR_MIX)
		lowp vec4 DF2 	= texture2D(diffuseMap2, uv2);
		DF = mix(DF,DF2, mixVal);
	#endif
	
	#if defined(USE_SPECULARLIGHT) || defined(USE_DIFFUSELIGHT) || defined(USE_NDIS) || defined(USE_NDISREFLECTION)
		#if defined(USE_VCOLOR_MIX)
				lowp vec4 NM 	= texture2D(normalMap , uv0);
				lowp vec4 NM2 	= texture2D(normalMap2, uv2);
				NM = mix(NM,NM2, mixVal) * 2.0 - 1.0;
		#else
			lowp vec4 NM = texture2D(normalMap , uv0) * 2.0 - 1.0;
		#endif
	#endif
	
	#if defined(USE_EMISSIVENESS) || defined(USE_SPECULARMASK) || defined(USE_VCOLOR_MASKB_TINT) || defined(USE_VCOLOR_MIX)
		lowp vec4 MK = texture2D(maskMap, uv0);	
	#endif
	
	#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS) || defined(USE_NDISREFLECTION)
		#if defined(USE_VCOLOR_MIX)
			lowp vec4 MK2 	= texture2D(maskMap2, uv2);
			MK = mix(MK,MK2, mixVal);
		#endif	
	#endif	
	
	#if defined(USE_VCOLOR_MASKB_TINT)
		DF.rgb = mix(DF.rgb,VC.rgb,MK.b);
	#endif
	
	#if defined(USE_DIFFUSELIGHT)
		//lowp float NdotL 	= max(dot(NM.rgb, normalize(LightDirection)),0.0);
		lowp float NdotL 	= max(dot(NM.rgb, LightDirection),0.0);
		#if defined(USE_LINEARSPACE_DF)
			//liniar space correction 
			NdotL = pow(NdotL, 0.45);
		#else
			//half lambertian space
			NdotL	= (NdotL + 1.0) * 0.5;
		#endif
		//lowp float DFL		= NdotL + ((1.0 - NdotL) * (1.0 - DFLIntensity));
		lowp float DFL		= mix(NdotL, 1.0, DFLIntensity);
	#endif
	
	#if defined(USE_NDIS) || defined(USE_SPECULARLIGHT) || defined(USE_NDISREFLECTION)
		lowp vec4 SP = vec4(0.0,0.0,0.0,0.0);
	#endif
	
	#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT0) // 0 = specPerLight
		mediump float NdotH 	= max(dot(NM.rgb, normalize(H)), 0.0);
		#if defined(USE_LINEARSPACE_SP) 
			//liniar space correction 
			NdotH = pow(NdotH, 0.45); 
		#endif
		SP = vec4(inPLightDF[0],1.0) * pow(NdotH,specularGlossy) * specularIntensity;
	#endif
	
	
	#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT1) // 1 = specPalettePerLight
		lowp float NdotH 	= dot(NM.rgb, normalize(H));
		SP		= texture2D(specularPalette , vec2(NdotH, 0.5));
		SP		= SP * vec4(inPLightDF[0], 1.0) * specularIntensity;
	#endif
	
	#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT2) // 2 = specPerView
		lowp vec3 VDir 		= normalize(ViewDirection);
		lowp float NdotV	= dot(NM.rgb, VDir);
		#if defined(USE_LINEARSPACE_SP) 
			//liniar space correction 
			NdotV = pow(NdotV, 0.45); 
		#endif
		SP = vec4(inPLightDF[0],1.0) * pow(NdotV,specularGlossy) * specularIntensity;
	#endif
	
	#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT3) // 3 = specPalettePerView
		lowp vec3 VDir 		= normalize(ViewDirection);
		lowp float NdotV 	= dot(NM.rgb, VDir);
		SP					= texture2D(specularPalette , vec2(NdotV, 0.5));
		SP					= SP * vec4(inPLightDF[0], 1.0) * specularIntensity;
	#endif
	
	#if defined(USE_NDIS) || defined(USE_NDISREFLECTION)
		lowp vec2 NDISuv = vec2((NM.x + VSN.x),(NM.y - VSN.y));
	#endif
	
	#if defined(USE_NDISREFLECTION)
		lowp vec4 REF = texture2D(reflectionMap, NDISuv.xy);
	#endif
	
	#if defined(USE_NDIS)
		SP = SP + texture2D(reflectionMap, NDISuv.xy) * reflectionAmount;
		DF.a = min(DF.a + SP.r,1.0); 
	#endif
	
	#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS)
		#if defined(USE_SPECULARMASK)
			lowp float specMask = MK.r;
		#else 
			// Generate a fake spec mask using diffuse luminance and some "luminance constants"
			lowp vec3 Lum = vec3( 0.2126, 0.7152, 0.0722);
			lowp float specMask = dot(Lum, DF.rgb);
		#endif
		
		#if defined (USE_VCOLOR_LM)
			SP = (SP * VC) * specMask;
		#else 
			SP = (SP * LM) * specMask;
		#endif
	#endif
	#if defined (USE_VCOLOR_LM)
			lowp vec4 VCOL = VC;
	#endif 
		
	#if defined(USE_EMISSIVENESS)
		#if defined (USE_VCOLOR_LM)
			VCOL = min(VC + MK.g, 1.0);
		#else 
			LM = min(LM + MK.g, 1.0);
		#endif
	#endif

	#if defined(USE_SNOWCOVERED)
		lowp vec4 snow	= texture2D(snowMap , uvSnow.rg);
		lowp vec4 edge	= texture2D(edgeMap , uvSnow.ba);
		lowp float mixAmount	= min(pow((edge.r * snowMask * snowIntensity),snowThreshold),1.0);
		DF						= mix(DF, snow,mixAmount);
	#endif
	
	#if defined(USE_NDISREFLECTION)
		DF = mix(DF , REF, reflectionAmount);
	#endif
	
	#if defined(USE_CH3DECALS)
		DF *= DECAL;
	#endif	
	
	#if defined(USE_CH3DECALSMIX)
		DF = mix(DF , DECAL, DECAL.a);
	#endif
	
	//COMPLETE
	#if defined (USE_VCOLOR_LM)
		lowp vec4 Complete = DF * VCOL * overbright;
	#else 
		lowp vec4 Complete = DF * LM + (LM * brightBurn) * overbright;
		//lowp vec4 Complete = DF * LM  * overbright;
	#endif

	#if defined(USE_DIFFUSELIGHT)
		Complete *= DFL;
	#endif
	
	#if defined(USE_VERTEXLIGHT)
		Complete += VXL;
	#endif
	
	#if defined(USE_SPECULARLIGHT) || defined(USE_NDIS)
		Complete += SP;
	#endif
	
	#if defined(USE_FOG)
		Complete = mix(Complete, inFogColor, fogRatio);
	#endif
	
	#if defined(USE_ANIMATEDOVERLAY)
		lowp vec4 OL		= texture2D(overlayMap , uvOverlay);
	#endif
	
	#if defined(USE_ANIMATEDOVERLAY_CAUSTIC)
		lowp vec2 uvCaustic		= (texture2D(causticNormalMap , uvOverlay).xy * 2.0 - 1.0) * distorsionAmount;
		lowp vec4 OL			= texture2D(overlayMap , (uv0 * overlayUVs.y)+uvCaustic);
	#endif
	
	#if defined(USE_ANIMATEDOVERLAY) || defined(USE_ANIMATEDOVERLAY_CAUSTIC)
		Complete += OL;	
	#endif
	
	#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
		//highp float depth = texture2D(shadowMap, lightProjectedUV).r;
		//if (depth < lightProjectedZ)
		//Complete *= max(vec3(0.6), Complete*0.8);
		lowp vec4 sh = texture2D(shadowMap, lightProjectedUV);
		sh = vec4(1.0) - sh * shadowMapOpacity;
		Complete *= sh;
	#endif
	
	lowp vec4 finalColor;
	
	#if defined(USE_CH3ALPHA) || defined (USE_VCOLOR_ALPHA)
		#if defined(USE_CH3ALPHA) 
			finalColor = vec4(Complete.rgb,ALPHA);
		#endif
		
		#if defined (USE_VCOLOR_ALPHA)
			finalColor = vec4(Complete.rgb,VC.r);
		#endif
	#else 
		finalColor = vec4(Complete.rgb,DF.a);
	#endif
	
#if defined(USE_COLORCORRECTION)	
	gl_FragColor = clamp(g_ColorCorrectionMatrix*finalColor,vec4(0),vec4(1));
	gl_FragColor.a = finalColor.a;
#else
	gl_FragColor = finalColor;
#endif
}