/*
==============================================================
Ubershader usable parameters:
-----------------------------
Guide:
   +	: optional parameter (the previous one should be added first)
   ++ 	: mandatory parameter (one of them must be added if the previous one is added first)
   -or-	: exclusive parameter (if the element it's added noone of the similar/neighbour params can be added)
-----------------------------
USE_FOG -> the shader use fog
	+ USE_VFOG -> the shader will use vertical fog instad of the classical fog
	
USE_EMISSIVENESS -> the G channel of the MK texture is the mask for the emissiven (self illuminated) parts of the object

USE_DIFFUSELIGHT -> adds diffuse-perPixelLighting (no specular, only diffuse -> NdotL)

USE_LINEARSPACE_DF -> corrects the NdotL calculation to linear-space (pow 0.45)

USE_SPECULARLIGHT -> adds specular light to the object
	++ USE_SPECULARLIGHT0 -> specPerLight (specular computed per light position + specularGlossiness)
	++ USE_SPECULARLIGHT1 -> specPalettePerLight (specular computed per light position + specularPalette [Fx_SPEC_xxxxx.tga])
	++ USE_SPECULARLIGHT2 -> specPerView (specular computed per camera + specularGlossiness)
	++ USE_SPECULARLIGHT3 -> specPalettePerView (specular computed per camera + specularPalette [Fx_SPEC_xxxxx.tga])
	
USE_LINEARSPACE_SP -> corrects the specular calculation to linear-space (pow 0.45)
	
USE_SPECULARMASK -> the R channel of the MK texture is the mask for the specular (specular level) of the objects. If this is not active, the specular mask is computed per luminance intensity of the diffuse texture.

USE_NDIS -> adds the NDIS texture as specular (thid means: additive over the diffuse). If there is specualr too, both are added.
 -or-
USE_NDISREFLECTION -> adds the NDIS texture as reflection (thid means: mix over the diffuse).
	+ USE_NDISREFRACTION -> compute the fake reflection as refraction for any of the two previous NDIS.
	
USE_CH3DECALS -> multiply a decalMap (mapped using the 3dsmax UVW-channel#3 / shader uv2) over the diffuse texture.
-or-
USE_CH3DECALSMIX -> mix by decal-alpha a decalMap (mapped using the 3dsmax UVW-channel#3 / shader uv2) over the diffuse texture.
-or-
USE_CH3ALPHA -> use the R channel of a texture called "alphaMap" (mapped with the uvw ch3) as alpha

USE_VCOLOR_MIX -> mix a second set of textures (DF2 + NM2 + MK2)
	++ USE_VCOLOR_MIX0	-> mix the second set of textures with the VC.r
	-or-
	++ USE_VCOLOR_MIX1 -> mix the second set of textures with a bitmap mask: (dirtMixMap)
-or-
USE_VCOLOR_MASKB_TINT -> multiply the vertex color over the diffuse modulated with the B channel of the MK texture
-or-
USE_VCOLOR_ALPHA -> use the R channel of then VC as alpha channel.
-or-
USE_VCOLOR_LM -> use the vertex color as LM

USE_SNOWCOVERED -> mix a snowMap using a mask texture (edgeMap). both textures use a custom uv map coordinates

USE_ANIMATEDOVERLAY -> add an animated overlay over the material
-or-
USE_ANIMATEDOVERLAY_CAUSTIC -> add an animated overlay distorted with a normalmap over the material

USE_VERTEXLIGHT -> use vertex lighting calculated with a maximum of 5 simultaneous lights. The ligh is applied with additive 

USE_SHADOWMAP ++ USE_SHADOWS -> use shadowmapping.

==============================================================
Define parameters (example):
----------------------------
#define USE_FOG	
#define USE_VFOG					//<-- optional for [USE_FOG]
#define USE_EMISSIVENESS
	//#define USE_DIFFUSELIGHT
#define USE_SPECULARLIGHT
#define USE_SPECULARLIGHT0			//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT1	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT2	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_SPECULARLIGHT3	//<-- mandatory for [USE_SPECULARLIGHT]
	//#define USE_LINEARSPACE_DF
	//#define USE_LINEARSPACE_SP
#define USE_SPECULARMASK
	//#define USE_NDIS				//<-- exclusive between [USE_NDIS] & [USE_NDISREFLECTION]
#define USE_NDISREFLECTION			//<-- exclusive between [USE_NDIS] & [USE_NDISREFLECTION]
	//#define USE_NDISREFRACTION 	//<-- optional for [USE_NDIS] or [USE_NDISREFLECTION]
	//#define USE_CH3DECALS			//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_CH3DECALSMIX		//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_CH3ALPHA			//<-- exclusive between [USE_CH3DECALS] & [USE_CH3DECALSMIX] & [USE_CH3ALPHA]
	//#define USE_VCOLOR_MIX		//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_MIX0		//<-- mandatory for [USE_VCOLOR_MIX]
	//#define USE_VCOLOR_MIX1		//<-- mandatory for [USE_VCOLOR_MIX]
	//#define USE_VCOLOR_MASKB_TINT	//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_ALPHA		//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_VCOLOR_LM			//<-- exclusive between [USE_VCOLOR_MIX] & [USE_VCOLOR_ALPHA] & [USE_VCOLOR_MASKB_TINT] & [USE_VCOLOR_LM]
	//#define USE_SNOWCOVERED
#define USE_ANIMATEDOVERLAY
	//#define USE_ANIMATEDOVERLAY_CAUSTIC //<-- optional for [USE_ANIMATEDOVERLAY]
#define USE_VERTEXLIGHT
#define USE_SHADOWMAP
#define USE_SHADOWS					//<-- mandatory for [USE_SHADOWMAP]
==============================================================
*/

#if defined(HAS_CURVATURE)
	#include "curvature.vs"
	uniform highp vec3 inCameraFront;
	uniform highp vec3 inCameraPos;
	uniform highp vec2 inCurvatureParams;

	uniform highp mat4 inViewProjMatrix;
	uniform highp mat4 inWorldMatrix;
#else //HAS_CURVATURE
	uniform mat4 inWorldViewProjMatrix;		
#endif //HAS_CURVATURE

attribute vec4 inPos;
attribute vec2 inUV0;


varying highp   vec2  uv0;

#if defined(USE_ANIMATEDOVERLAY) || defined(USE_ANIMATEDOVERLAY_CAUSTIC)
	uniform highp float fracTime;
	uniform highp vec2 overlayUVs;
	uniform highp vec2 speed;
	varying highp	vec2 uvOverlay;
#endif

#if !defined (USE_VCOLOR_LM)
	attribute vec2 inUV1;
	varying highp   vec2  uv1;
#endif

#if defined(USE_VCOLOR_MIX) || defined(USE_CH3ALPHA) || defined(USE_CH3DECALS) || defined (USE_SNOWCOVERED)
	attribute vec2 inUV2;
#endif

#if defined(USE_VCOLOR_MIX) || defined(USE_CH3ALPHA) || defined(USE_CH3DECALS)
	varying highp   vec2  uv2;
#endif

#if defined(USE_VCOLOR_MIX) && defined(USE_VCOLOR_MIX1)
	uniform highp 	 float dirtMixUVMultiplier;
	varying highp    vec2  uvDirt;
#endif

#if defined(USE_VERTEXLIGHT) || defined(USE_SPECULARLIGHT) || defined(USE_NDIS) || defined (USE_SNOWCOVERED) || defined(USE_NDISREFLECTION) || defined(HAS_SKINNING)
	attribute vec4 inNormal;
#endif

#if defined(USE_VERTEXLIGHT) || defined(USE_SPECULARLIGHT) || defined(HAS_SKINNING)
	attribute vec4 inTangent;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined(USE_VCOLOR_MIX) || defined (USE_VCOLOR_ALPHA) || defined (USE_VCOLOR_LM)
	attribute vec4 	inColors;
	varying lowp 	vec4  	VC;
#endif

#if defined(USE_SNOWCOVERED)
	uniform	mediump float snowUVMultip;
	varying highp   vec4  uvSnow;
	varying lowp   	float  snowMask;
#endif

#if defined(USE_FOG)
	uniform	mediump float 		inFogStart;
	uniform	mediump float 		inFogRangeI;
	uniform lowp 	vec4 		inFogColor;				
	varying lowp 	float 		fogRatio;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT0) || defined(USE_SPECULARLIGHT1)
	varying highp 	vec3 H;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT2) && defined(USE_SPECULARLIGHT3)
	varying highp vec3 	ViewDirection;
#endif

#if defined(USE_SPECULARLIGHT) || defined(USE_VERTEXLIGHT)
	uniform highp 	vec3 	inPLightPos[8];
	uniform highp 	vec3 	inCameraPosOS;
#endif

#if defined(USE_DIFFUSELIGHT)
	varying highp 	vec3  	LightDirection;
#endif

#if defined(USE_VERTEXLIGHT)
	uniform lowp 		vec3 	lightDiffuse[5];
	uniform mediump 	float 	lightRadius[5];
	varying lowp 		vec4	VXL;
#endif		

#if defined(USE_NDIS) || defined(USE_NDISREFLECTION)
	varying highp vec2  VSN;
	uniform highp float reflectionBend;
	uniform mat4 inWorldViewITMatrix;
#endif

#if defined(USE_VFOG)
	uniform	mediump float 	inFogHeigh;
#endif

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	uniform highp mat4 inLightTextureMatrix;
	varying highp vec2 lightProjectedUV;
	//varying highp float lightProjectedZ;	
	#if !defined(HAS_CURVATURE)
		uniform highp mat4 inWorldMatrix;
	#endif //	
#endif

void main()
{
	#if defined(HAS_CURVATURE)
		highp vec4 posW = inWorldMatrix * vec4(inPos.xyz, 1.0);
		posW = computeCurvature(posW, inCameraPos, inCameraFront, inCurvatureParams.x, inCurvatureParams.y);
		gl_Position = inViewProjMatrix * posW;
	#else //HAS_CURVATURE
		gl_Position = inWorldViewProjMatrix * vec4(inPos.xyz, 1.0);
	#endif //HAS_CURVATURE

uv0 = inUV0;

#if defined(USE_ANIMATEDOVERLAY) || defined(USE_ANIMATEDOVERLAY_CAUSTIC)
	highp vec2 uvOffset = vec2(speed) * fracTime;
	uvOverlay = (uv0 * overlayUVs.x) + uvOffset;
#endif

#if !defined (USE_VCOLOR_LM)
	uv1 = inUV1;
#endif

#if defined(USE_SHADOWMAP) && defined(USE_SHADOWS)
	highp mat4 lm = inLightTextureMatrix * inWorldMatrix; //this is detected by the GPU and precomputed
	highp vec3 lightCoord  = (lm * vec4(inPos.xyz, 1.0)).xyz;
	lightProjectedUV = lightCoord.xy;
	//lightProjectedZ = min(lightCoord.z + 0.0003, 1.0);
#endif

#if defined(USE_SNOWCOVERED)
	uvSnow.rg = inUV0 * snowUVMultip;
	uvSnow.ba = inUV2 * snowUVMultip;
	snowMask = max(inNormal.b,0.0);
#endif
#if defined(USE_VCOLOR_MIX) && defined(USE_VCOLOR_MIX1)
	uvDirt = inUV0 * dirtMixUVMultiplier;
#endif

#if defined(USE_VCOLOR_MIX) || defined(USE_CH3ALPHA) || defined(USE_CH3DECALS)
	uv2 = inUV2;
#endif

#if defined(USE_VCOLOR_MASKB_TINT) || defined(USE_VCOLOR_MIX) || defined (USE_VCOLOR_ALPHA) || defined (USE_VCOLOR_LM)
	VC = inColors;
#endif

#if defined(USE_NDIS) || defined(USE_NDISREFLECTION)
	highp vec2 fragPos 	= normalize(gl_Position.xyz).xy;
	#if defined(USE_NDISREFRACTION)
		highp vec3 vsn1	= (inWorldViewITMatrix *  vec4(-inNormal.xyz, 0.0)).xyz;
		VSN 	= normalize(vsn1).xy * 0.5 + 0.5;
		VSN 	-= fragPos * reflectionBend;
	#else
		highp vec3 vsn1	= (inWorldViewITMatrix *  vec4(inNormal.xyz, 0.0)).xyz;
		VSN 	= normalize(vsn1).xy * 0.5 + 0.5;
		VSN 	+= fragPos * reflectionBend;
	#endif
#endif

#if defined(USE_VERTEXLIGHT) || defined(USE_SPECULARLIGHT)
	highp vec3 Normal   = normalize(inNormal.xyz);
	highp vec3 Tangent  = normalize(inTangent.xyz);
	highp vec3 Binormal = cross(Normal, Tangent) * -inTangent.w;
#endif

#if defined(USE_SPECULARLIGHT) && defined(USE_SPECULARLIGHT2) && defined(USE_SPECULARLIGHT3)
	highp vec3 ViewDir  = normalize(inCameraPosOS - inPos.xyz);
	ViewDirection.x = dot( Tangent, ViewDir );
	ViewDirection.y = dot( Binormal, ViewDir );
	ViewDirection.z = dot( Normal, ViewDir );
	ViewDirection 	= normalize(ViewDirection);
#endif

#if defined(USE_SPECULARLIGHT0) || defined(USE_SPECULARLIGHT1)
	highp vec3 ViewDir  = normalize(inCameraPosOS - inPos.xyz);
	highp vec3 ViewDirec; 
	ViewDirec.x = dot( Tangent, ViewDir );
	ViewDirec.y = dot( Binormal, ViewDir );
	ViewDirec.z = dot( Normal, ViewDir );
	ViewDirec 	= normalize(ViewDirec);
#endif

#if defined(USE_DIFFUSELIGHT) || defined(USE_VERTEXLIGHT) || defined(USE_SPECULARLIGHT)
	highp vec3 LightDir = normalize(inPLightPos[0].xyz - inPos.xyz);
#endif

#if defined(USE_DIFFUSELIGHT)
	LightDirection.x = dot( Tangent, LightDir );
	LightDirection.y = dot( Binormal, LightDir );
	LightDirection.z = dot( Normal, LightDir );
	LightDirection 	= normalize(LightDirection);
#endif

#if defined(USE_VERTEXLIGHT) || defined(USE_SPECULARLIGHT)
	highp vec3 LightDirec;
	LightDirec.x = dot( Tangent, LightDir );
	LightDirec.y = dot( Binormal, LightDir );
	LightDirec.z = dot( Normal, LightDir );
	LightDirec 	= normalize(LightDirec);
#endif

#if defined(USE_VERTEXLIGHT)
	lowp vec4 VL = vec4(0.0,0.0,0.0,1.0);
	for(int i = 1; i <= 5; i++) {
		lowp float R 		= lightRadius[i-1];
		lowp float D 		= length(inPLightPos[i] - inPos.xyz);
		lowp float Att 		= 1.0 - clamp(D/R, 0.0, 1.0);
		
		lowp vec3 L  		= normalize(inPLightPos[i] - inPos.xyz);
		lowp float NdotL 	= pow(dot(Normal, L),0.45);
		NdotL 				= max(NdotL, 0.0) * Att;
		VL				  	= VL + (NdotL * vec4(lightDiffuse[i-1],1.0));
	}
	VXL = VL;
#endif

#if  defined(USE_SPECULARLIGHT) && (defined(USE_SPECULARLIGHT0)) || (defined(USE_SPECULARLIGHT1))
	H = normalize(LightDirec + ViewDirec);
#endif			

#if defined (USE_FOG)
	lowp float fogRatioY = 0.0;
	
	#if defined (USE_VFOG)
		fogRatioY = clamp(posW.z - inFogHeigh, 0.0, 1.0);
	#endif
	
	fogRatio = clamp((gl_Position.z - inFogStart) * inFogRangeI, 0.0, 1.0) * inFogColor.a * (1.0 - fogRatioY);
#endif
}
